;DC20.ASM - Source code to clock program which will display current time in
;12-hour format in graphic digits in the middle of the screen-this is an update
;to DIGCLOCK.ASM.

;DIGCLOCK.ASM - DIGCLOCK      - first release
;DC20.ASM     - DIGCLOCK v2.0 - now supports CGA, EGA, VGA


CODE		SEGMENT
	ORG	0h			;THIS WILL BE AN EXE FILE
	ASSUME	CS:CODE,DS:DATA,SS:STACK ;PROPER SETUP FOR THE EXE FILE

MAIN	PROC	NEAR			;MAIN PROCEDURE


	MOV	AX,DATA			;DS TO POINT TO DATA SEGMENT
	MOV	DS,AX

	MOV	AH,1Ah			;FIRST OFF, CHECK FOR VGA
	MOV	AL,0
	INT	10h
	CMP	AL,1Ah			;IF THIS FUNCTION SUPPORTED-AL=1Ah
	JNZ	NO_VGA			;NO VGA? TRY EGA
	CMP	BL,8			;IF VGA, BL = 8
	JNZ	NO_VGA			;CLOSE BUT NO CIGAR, TRY EGA
	MOV	AX,0013h		;ALRIGHT! SET VGA MODE 13h 320X200X256X256 COLORS
	INT	10h			;
	MOV	BYTE PTR MAX_COLOR,255	;HIGHEST COLOR # WE CAN DISPLAY
	JMP	CONT_WITH_SETTING	;AND GET THE CLOCK GOING
NO_VGA:
	MOV	AH,12h			;WELL, DO WE HAVE AN EGA?
	MOV	BL,10h			;IF BL NOT 10h AFTER THIS, WE HAVE EGA
	INT	10h			;HERE GOES
	CMP	BL,10h			;SURVEY SAID . . .
	JNZ	EGA_PRESENT		;DING DING DING! EGA!

					;A CGA? CURSES! OH WELL
	MOV	AX,0004h		;SET CGA MODE 4 - 320X200X4 COLORS
	INT	10h
	MOV	BYTE PTR MAX_COLOR,3
	JMP	CONT_WITH_SETTING	;AND GET THE CLOCK UNDERWAY
EGA_PRESENT:
	MOV	AX,000Dh		;FOUND AN EGA, REMEMBER?
	INT	10h			;WE'RE GONNA USE IT
	MOV	BYTE PTR MAX_COLOR,15
CONT_WITH_SETTING:
	CALL	SET_CLOCK		;SET THE CLOCK
RUN:

	CALL	DISP			;DISPLAY TIME
	CALL	NC			;INCREMENT TIME
	CALL	PAUSE			;WAIT 1 SECOND
	CMP	BYTE PTR ENDFLAG,0FFh	;TIME TO END?
	JZ	TERMINATE		;YES,EXIT
	MOV	AH,1			;POLL KEYBOARD
	INT	16h			;DO IT
	JZ	RUN			;ZR = NO KYSTROK,CONTINUE RUNNING

TERMINATE:
	MOV	AX,0003h		;SET MODE BACK TO 80X25X16 TEXT
	INT	10h
	MOV	AX,4C00h		;LEAVE,EXIT CODE=0
	INT	21h			;EXIT FROM STAGE RIGHT

MAIN	ENDP

PAUSE	PROC	NEAR

	PUSH	AX
	PUSH	DX
	PUSH	CX

WAIT:
	MOV	AH,2Ch		;GET TIME
	INT	21h
	CMP	DH,OLDDH	;ARE SECONDS SAME?
	JZ	WAIT		;YES,1 SECOND NOT PASSED YET	
	MOV	OLDDH,DH	;NO,SECOND PASSED,UPDATE SECONDS IN OLD BUFFER

	CALL	CHANGE_COLOR	;CHANGE THE COLOR OF NEXT DISPLAY
	POP	CX
	POP	DX
	POP	AX

	RET			;LEAVE - ONE MORE TICK HAS PASSED

PAUSE	ENDP
CHANGE_COLOR	PROC	NEAR
	PUSH	AX
	MOV	AH,BYTE PTR MAX_COLOR		;AH=HIGHEST COLOR DISPLAYABLE
	MOV	AL,BYTE PTR CURRENT_COLOR	;AL=CURRENT COLOR
	INC	AL				;INCREMENT COLOR
	CMP	AL,AH				;IS IT AT LIMIT
	Jbe	NOT_OVER			;NOPE
	MOV	AL,1				;YEP! SET BACK TO ZERO
NOT_OVER:
	MOV	BYTE PTR CURRENT_COLOR,AL	;AT ANY RATE, ADJUST COLOR
	POP	AX
	RET					;AND BUST OUT OF HERE
CHANGE_COLOR	ENDP

DISP_NUMBER	PROC	NEAR

	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX

	PUSH	SI
	PUSH	DI


	ADD	AL,AL		;DOUBLE AL - OFFSETS OF NUMBERS ARE WORDS
	MOV	SI,OFFSET TABLE	;OFFSET OF TABLE POINTING OUT NUMBERS
	XOR	AH,AH		;HIGH HALF OF AX = 0
	ADD	SI,AX		;GET OFFSET INTO TABLE

	MOV	BX,SI		;GET OFFSET OF NUMBER TO PRINT
	MOV	SI,WORD PTR [BX] ;OFFSET NOW IN SI

	MOV	STARTX,CX	;X POSITION OF NUMBER
	MOV	STARTY,DX	;Y POSITION OF NUMBER
	MOV	OFFX,0		;X-POSITION RELATIVE FROM STARTX
	MOV	OFFY,0		;Y-POSITION RELATIVE FROM STARTY

DRAW:
	MOV	AH,0Ch		;0CH=BIOS PLOT PIXEL SERVICE
	MOV	AL,[SI]		;COLOR:COLOR OF POINT IN # PATTERN = 1
	XOR	BH,BH		;PAGE 0 OF VIDEO MEMORY
	MOV	CX,STARTX	;X POSITION WE STARTED FROM
	MOV	DX,STARTY	;Y POSITION WE STARTED FROM
	ADD	CX,OFFX		;OFFSET FROM X ON SCREEN
	ADD	DX,OFFY		;OFFSET FROM Y ON SCREEN

	CMP	AL,1		;IS THIS A SPOT TO LIGHT UP?
	JNZ	AL_IS_ZERO	;NOPE, THEN IT'S TO STAY DARK:CONTINUE
	MOV	AL,BYTE PTR CURRENT_COLOR	;GET CURRENT COLOR INTO AL
AL_IS_ZERO:
	INT	10h		;BIOS VIDEO CALL

	INC	SI		;BUMP POINTER INTO NUMBER PATTERN

	INC	OFFX		;BUMP LOCATION OF PLOT POINT ON SCREEN
	CMP	OFFX,9		;GONE PAST 9?
	JLE	DRAW		;NO,CONTINUE DRAWING NUMBER
	MOV	OFFX,0		;YES,MOVE BACK TO X STARTING POINT
	INC	OFFY		;BUT THIS TIME NEXT LINE OF # WILL BE DONE
	CMP	OFFY,9		;UNLESS WE HAVE FINISHED THE LAST ONE . . .
	JLE	DRAW		;IF NOT CONTINUE . . . 
	MOV	OFFY,0		;ELSE WE ARE THROUGH
	POP	DI		;IN WHICH CASE WE'LL LEAVE
	POP	SI

	POP	DX
	POP	CX
	POP	BX
	POP	AX
	RET

DISP_NUMBER	ENDP

NC	PROC	NEAR

	INC	BYTE PTR [CTR+5]	;INCREMENT ONES PLACE OF SECONDS
	CMP	BYTE PTR [CTR+5],9	;OVER 9?
	JLE	COK			;NOPE,CAN JUST RETURN
	MOV	BYTE PTR [CTR+5],0	;BACK TO ZERO
	INC	BYTE PTR [CTR+4]	;INCREMENT TENS PLACE OF SECONDS
	CMP	BYTE PTR [CTR+4],5	;OVER 5? (END OF THIS MINUTE?)
	JLE	COK			;NO,OK
	MOV	BYTE PTR [CTR+4],0	;YES,ZERO TENS DIGIT
	INC	BYTE PTR [CTR+3]	;INCREMENT MINUTE ONES DIGIT
	CMP	BYTE PTR [CTR+3],9	;OVER 9?
	JLE	COK			;NO,OK
	MOV	BYTE PTR [CTR+3],0	;ZERO IT
	INC	BYTE PTR [CTR+2]	;INCREMENT TENS PLACE OF MINS.
	CMP	BYTE PTR [CTR+2],5	;OVER 5? (END OF HOUR?)
	JLE	COK			;NOPE,OK		
	MOV	BYTE PTR [CTR+2],0	;ZERO IT
	INC	BYTE PTR HR		;INCREMENT SPECIAL HOUR COUNTER
	CMP	BYTE PTR HR,12		;DID IT JUST STRIKE 12?
	JNZ	NOT_TWELVE		;NOPE,NO AM/PM CHANGE
	CALL	TOGGLE_AMPM		;YES,CHANGE FROM AM->PM OR VICE VERSA
NOT_TWELVE:
	CMP	BYTE PTR HR,12		;IS IT PAST 12?
	JLE	JUST_INC_HOUR		;NOPE,CONTINUE
	MOV	BYTE PTR HR,1		;YES,GO BACK TO 1 O'CLOCK
	MOV	BYTE PTR [CTR+0],0	;ZERO TEN DIGIT OF HOURS
	MOV	BYTE PTR [CTR+1],1	;SET ONE DIGIT OF HOURS TO 0

	JMP	COK			;CHECKS OK
JUST_INC_HOUR:
	INC	BYTE PTR [CTR+1]	;NOT 12 OR 1 O'CLOCK,JUST INC HOUR
	CMP	BYTE PTR [CTR+1],9	;PAST NINE?
	JLE	COK			;NOPE
	MOV	BYTE PTR [CTR+1],0	;YEP,SO ZERO ONES DIGIT
	INC	BYTE PTR [CTR+0]	;AND INCREMENT TENS DIGIT
COK:

	RET

NC	ENDP

DISP	PROC	NEAR

	MOV	AL,BYTE PTR [CTR+0]	;TENS DIGIT HOURS
	CMP	AL,0			;ZERO?
	JNZ	DISPLAY_AL		;NO,MUST DISPLAY
	MOV	AL,10			;YES,POINT TO BLANK(DON'T DISPLAY
					;UNNECESSARY EXTRA ZERO)
DISPLAY_AL:
	MOV	CX,100			;X POSITION ON SCREEN
	MOV	DX,80			;Y POSITION ON SCREEN
	CALL	DISP_NUMBER		;ZAP IT ON SCREEN
	MOV	AL,BYTE PTR [CTR+1]	;ONES DIGIT HOURS
	MOV	CX,112			;X
	MOV	DX,80			;Y
	CALL	DISP_NUMBER		;ZAP IT
	MOV	AL,BYTE PTR [CTR+2]	;TENS DIGIT MINS
	MOV	CX,140			;X
	MOV	DX,80			;Y
	CALL	DISP_NUMBER		;ZAP IT
	MOV	AL,BYTE PTR [CTR+3]	;ONES DIGIT MINS
	MOV	CX,152			;X
	MOV	DX,80			;Y
	CALL	DISP_NUMBER		;ZAP IT
	MOV	AL,BYTE PTR [CTR+4]	;TENS DIGIT SECS
	MOV	CX,180			;X
	MOV	DX,80			;Y
	CALL	DISP_NUMBER		;ZAP IT
	MOV	AL,BYTE PTR [CTR+5]	;ONES DIGIT SECS
	MOV	CX,192			;X
	MOV	DX,80			;Y
	CALL	DISP_NUMBER		;ZAP IT
	CALL	DISP_AMPM		;DISPLAY AM/PM INDICATOR DOT
	RET
DISP	ENDP

SET_CLOCK	PROC	NEAR

	MOV	AH,2Ch			;GET TIME
	INT	21h
	CMP	CH,12			;12 O'CLOCK PM?
	JNZ	NOT_NOON		;NOPE	
	MOV	BYTE PTR AMPM,1		;YES,SET INDICATOR
NOT_NOON:

	CMP	CH,12			;12 PM OR PAST 12 PM?
	JLE	DONT_FIX		;NO,DON'T NEED TO FIX HR FOR 12HR DISPLAY
	SUB	CH,12			;YES,ADJUST FOR 12 - HOUR DISPLAY
	MOV	BYTE PTR AMPM,1		;AND SET AM-PM DOT FOR PM

DONT_FIX:
	CMP	CH,0			;MIDNIGHT?
	JNZ	NOT_MIDNIGHT		;NO,HOUR WILL LOOK OK
	MOV	CH,12			;YES,MUST SET HOUR TO 12
NOT_MIDNIGHT:

	MOV	BYTE PTR HR,CH		;SPECIAL HOUR BYTE,HELPS SPECIAL
					;HOUR INCREMENTING PROCESS GO MUCH
					;EASIER

SET_HR:
	CMP	CH,0			;IS IT ZERO YET?
	JZ	SET_MIN			;YES,SO SET MINUTES		
	DEC	CH			;NO DECREMENT IT
	INC	BYTE PTR [CTR+1]	;AND CHALK UP ANOTHER HOUR
	CMP	BYTE PTR [CTR+1],9	;10'S DIGIT >9?
	JLE	HOK			;NOPE,OK		
	MOV	BYTE PTR [CTR+1],0	;ZERO IT
	INC	BYTE PTR [CTR+0]	;INCREMENT TENS DIGIT
HOK:	JMP	SET_HR			;CONTINUE SETTING HOURS

					;SAME PROCESS USED FOR SETTING
					;MINUTES AND SECONDS

SET_MIN:
	CMP	CL,0
	JZ	SET_SEC
	DEC	CL
	INC	BYTE PTR [CTR+3]
	CMP	BYTE PTR [CTR+3],9
	JLE	MOK
	MOV	BYTE PTR [CTR+3],0
	INC	BYTE PTR [CTR+2]
MOK:
	JMP	SET_MIN
SET_SEC:
	CMP	DH,0
	JZ	ALL_SET
	DEC	DH
	INC	BYTE PTR [CTR+5]
	CMP	BYTE PTR [CTR+5],9
	JLE	SOK
	MOV	BYTE PTR [CTR+5],0
	INC	BYTE PTR [CTR+4]
SOK:
	JMP	SHORT SET_SEC
ALL_SET:
	RET

SET_CLOCK	ENDP

TOGGLE_AMPM	PROC	NEAR


	CMP	BYTE PTR AMPM,0		;AM?
	JNZ	SET_AM			;NOPE,SO MUST BE SET TO AM
	MOV	BYTE PTR AMPM,1		;YES,SO MUST BE SET TO PM

	JMP	TOGGLE_DONE		;LEAVE
SET_AM:
	MOV	BYTE PTR AMPM,0		;SET TO AM
TOGGLE_DONE:
	RET
TOGGLE_AMPM	ENDP

DISP_AMPM	PROC	NEAR

	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX

	CMP	BYTE PTR AMPM,0		;AM?
	JNZ	DISP_PM			;NOPE,PM
	MOV	AH,0Ch			;SET AM PIXEL
	MOV	AL,2			;MAGENTA
	MOV	CX,95			;X POSITION
	MOV	DX,80			;Y POSITION
	INT	10h			;DO IT
	MOV	AH,0Ch			;SET PM PIXEL
	MOV	AL,0			;BLACK (OFF)
	MOV	CX,95			;X
	MOV	DX,90			;Y
	INT	10h			;DO IT
	JMP	DDONE			;DOT SETTING DONE
DISP_PM:
	MOV	AH,0Ch			;PM,SET AM PIXEL
	MOV	AL,0			;TO BLACK (OFF)
	MOV	CX,95			;X
	MOV	DX,80			;Y
	INT	10h			;DO IT
	MOV	AH,0Ch			;SET PM PIXEL
	MOV	AL,2			;MAGENTA
	MOV	CX,95			;X
	MOV	DX,90			;Y
	INT	10h			;DO IT
DDONE:
	POP	DX
	POP	CX
	POP	BX
	POP	AX

	RET
DISP_AMPM	ENDP

CODE		ENDS

DATA		SEGMENT

TABLE	DW	N0,N1,N2,N3,N4,N5,N6,N7,N8,N9,BLANK	;OFFSETS OF NUMBER
							;PATTERNS IN CODE
							;SEGMENT
CTR	DB	0,0,0,0,0,0				;COUNTER(TIME)
HR	DB	0					;HOUR
STARTX	DW	0					;POSITION OF # - X
STARTY	DW	0					;POSITION OF # - Y
OLDDH	DB	0					;CURRENT SECOND
							;WHEN THIS DOES NOT
							;MATCH CURRENT SECOND
							;GIVEN BY DOS,ANOTHER
							;SECOND HAS PASSED -
							;TIME FOR ANOTHER TICK
ENDFLAG	DB	0					;IF 0FFh,PROGRAM ENDS
OFFX  	DW	0					;X-POS OF NEXT PLOT
OFFY  	DW	0					;Y POS OF NEXT PLOT
AMPM	DB	0					;AM/PM INDICATOR SWITCH
MAX_COLOR	DB	0				;MAXIMUM COLOR # WE CAN
							;DISPLAY-FILLED IN AT
							;RUN-TIME
CURRENT_COLOR	DB	0				;COLOR TIME WILL BE
							;DISPLAYED IN-CHANGES
;HERE THE NUMBER PATTERNS IN THE FORM OF DATA BYTE BLOCKS OF 10X10
;ARE STORED FOR EASY FUTURE MODIFICATION. SOME INTERESTING NUMBER
;FONTS COULD BE CREATED BY DIGGING INTO THIS TABLE AND CHANGING IT.

N1	DB	0,0,1,1,1,1,1,0,0,0
	DB	0,0,1,1,1,1,1,0,0,0
	DB	0,0,0,0,1,1,1,0,0,0
	DB	0,0,0,0,1,1,1,0,0,0
	DB	0,0,0,0,1,1,1,0,0,0
	DB	0,0,0,0,1,1,1,0,0,0
	DB	0,0,0,0,1,1,1,0,0,0
	DB	0,0,0,0,1,1,1,0,0,0
	DB	0,0,0,0,1,1,1,0,0,0
	DB	0,0,0,0,1,1,1,0,0,0


N2	DB	0,1,1,1,1,1,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	0,0,0,0,0,0,0,1,1,1
	DB	0,1,1,1,1,1,1,1,1,1
	DB	1,1,1,1,1,1,1,1,1,0
	DB	1,1,1,0,0,0,0,0,0,0
	DB	1,1,1,0,0,0,0,0,0,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1


N3	DB	0,1,1,1,1,1,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	0,0,0,0,0,0,0,1,1,1
	DB	0,0,0,0,1,1,1,1,1,0
	DB	0,0,0,0,1,1,1,1,1,0
	DB	0,0,0,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,1,1,1,1,1,1,1,1,0


N4	DB	0,0,0,0,0,1,1,1,1,0
	DB	0,0,0,0,1,1,1,1,1,0
	DB	0,0,0,1,1,1,1,1,1,0
	DB	0,0,1,1,1,0,1,1,1,0
	DB	0,1,1,1,0,0,1,1,1,0
	DB	1,1,1,0,0,0,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,0,0,0,0,0,1,1,1,0
	DB	0,0,0,0,0,0,1,1,1,0

N5	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,0,0,0
	DB	1,1,1,1,1,1,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,0,0,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,1,1,1,1,1,1,1,1,0

N6	DB	0,1,1,1,1,1,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,0,0,0
	DB	1,1,1,1,1,1,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,1,1,1,1,1,1,1,1,0

N7	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	0,0,0,0,0,0,1,1,1,1
	DB	0,0,0,0,0,1,1,1,1,0
	DB	0,0,0,0,1,1,1,1,0,0
	DB	0,0,0,1,1,1,1,0,0,0
	DB	0,0,1,1,1,1,0,0,0,0
	DB	0,1,1,1,1,0,0,0,0,0
	DB	1,1,1,1,0,0,0,0,0,0

N8	DB	0,1,1,1,1,1,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	0,1,1,1,1,1,1,1,1,0
	DB	0,1,1,1,1,1,1,1,1,0
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,1,1,1,1,1,1,1,1,0

N9	DB	0,1,1,1,1,1,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,1,1,1,1,1,1,1,1,1
	DB	0,0,0,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,1,1,1,1,1,1,1,1,0

N0	DB	0,1,1,1,1,1,1,1,1,0
	DB	1,1,1,1,1,1,1,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,0,0,0,0,1,1,1
	DB	1,1,1,1,1,1,1,1,1,1
	DB	0,1,1,1,1,1,1,1,1,0

;THIS IS A BLANK BLOCK. WHEN THE HOURS ARE LESS THAN 10,THE TENS DIGIT
;OF THE HOURS DOESN'T NEED TO BE DISPLAYED - REAL 12-HOUR CLOCKS USUALLY 
;DON'T DISPLAY AN EXTRA ZERO AT THE BEGINNING WHEN IT IS NOT NEEDED.

BLANK	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0
	DB	0,0,0,0,0,0,0,0,0,0

NOTICE	DB	"Written by Darryl Seamans - 3/4/91 and updated 7/15/91"

DATA	ENDS


STACK	SEGMENT PARA STACK

	DW	160 DUP (0FFFFh)	;160 WORD-DEEP STACK

STACK	ENDS

	END	MAIN
